/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef BAIDUVISIONENGINE_P_H
#define BAIDUVISIONENGINE_P_H

#include <functional>
#include <string>
#include <kylin-ai/ai-engine/large-model/cloudvisionengine.h>
#include <kylin-ai/ai-engine/large-model/error.h>

class BaiduVisionEnginePrivate : public ai_engine::lm::vision::AbstractCloudVisionEngine {
public:
    BaiduVisionEnginePrivate();
    ~BaiduVisionEnginePrivate();
    std::string engineName() const override;
    int maxConcurrentPrompt2ImageTasks() const override;

    void setConfig(const std::string &config) override;
    std::string modelInfo() const override;

    void setPrompt2ImageCallback(ai_engine::lm::vision::ImageResultCallback callback) override;
    void setPrompt2ImageSize(int width, int height) override;
    void setPrompt2ImageNumber(int number) override;
    void setPrompt2ImageStyle(int style) override;
    void setPrompt2ImageStyle(const std::string &style) override;
    std::string supportedPrompt2ImageParams() const override;

    bool prompt2Image(const std::string &prompt, ai_engine::lm::EngineError &error) override;

private:
    std::string apiKey_;
    std::string secretKey_;
    std::string accessToken_;
    std::string &getAccessToken_();
    bool generateAccessToken_();

    ai_engine::lm::vision::ImageResultCallback imageResultCallback_;

    int imageWidth_{1024};
    int imageHeight_{1024};
    int imageNumber_{1};
    std::string imageStyle_;

    std::string requestPrompt2ImageTask(const std::string &prompt);
    bool processPrompt2ImageTask(const std::string &taskId, ai_engine::lm::EngineError &error);
    std::string doRequestTaskResult(const std::string &taskId);

    void runCallbackWithError(const ai_engine::lm::EngineError &error, const int &index, const int &total);
    ai_engine::lm::EngineError currentError_;
};

#endif // BAIDUVISIONENGINE_P_H
